@@ -1210,6 +1210,14 @@ auto search16_qprep(s16info_s * s, char * qseq, int qlen) -> void
     }
 }
 
+
+/*
+  Turn off tree-partial-pre optimizations for the rest of the file.
+  GNU C++ 9 or later generates incorrect code on x86_64 if turned on.
+*/
+
+#pragma GCC optimize ("-fno-tree-partial-pre")
+
 auto search16(s16info_s * s,
               unsigned int sequences,
               unsigned int * seqnos,
